"""
DEPRECATED: Use openhcs.core.lazy_placeholder_simplified instead.

This file is kept for backward compatibility but will be removed.
"""

# Redirect to simplified implementation
from openhcs.core.lazy_placeholder_simplified import *
